  //
  // This is an example Chromecast sender app to test with ExpressPlay using PlayReady tokens
  //
  // Use the application ID of your own receiver app.
  var applicationID = 'AC5DAA1F';
  var MESSAGE_NAMESPACE = 'urn:x-cast:com.expressplay.sample.mediaplayer';
  var currentMedia = null;
  var progressFlag = 1;
  var mediaCurrentTime = 0;
  var session = null;
  var initialTimeIndexSeconds = 0;
  var timer = null;
  var drmtype = 'pr',
      licenseUrl = 'http://pr.test.expressplay.com/playready/RightsManager.asmx',
      pause = '<i id="pause" class="fa fa-pause" aria-hidden="true">',
      play = '<i id="play" class="fa fa-play" aria-hidden="true">',
      resume = '<i id="resume" class="fa fa-play" aria-hidden="true">',
      pr_content = 'http://content.intertrust.com/dash/playreadymarlin/elephantdream1/stream.mpd',
      pr_license_req = 'https://content-access.intertrust-dev.com:1443/MIXEDDRM001/plr',
      wv_content = 'https://ms3-test.intertrust.com:8443/WideVineTest/wvmarlin/stream.mpd',
      wv_license_req = 'https://content-access.intertrust-dev.com:1443/MIXEDDRM003/wv';


  // Check to make sure the Chromecast function is only used in Chrome.
  var is_chrome = navigator.userAgent.toLowerCase().indexOf('chrome') > -1;
  $('#loading').hide();

  document.getElementById("drm_type").onchange = function(){
    toggleLoadBtn('disable');
    changeService(this.value);
  };

  function toggleLoadBtn(status){
    if(status === 'disable'){
      $('#load').hide();
      $('#loading').show();
      $('#loadMedia').attr('disabled', true);
      $('#playpauseresume').attr('disabled', true);
      $('#stop').attr('disabled', true);
    } else {
      $('#load').show();
      $('#loading').hide();
      $('#loadMedia').attr('disabled', false);
      $('#playpauseresume').attr('disabled', false);
      $('#stop').attr('disabled', false);
    }
  }

  function changeService(val){
    if(val === 'pr') {
      document.getElementById('mediaUrl').value = pr_content;
      document.getElementById('license_label').style.display = 'inline-block';
      document.getElementById('licenseUrl').style.display = 'inline-block';
      drmtype = 'pr';
      getRequest(val, pr_license_req);
    } else if(val === 'wv'){
      document.getElementById('mediaUrl').value = wv_content;
      document.getElementById('license_label').style.display = 'none';
      document.getElementById('licenseUrl').style.display = 'none';
      drmtype = 'wv';
      getRequest(val, wv_license_req);
    }
  }

  function getRequest(val, license){
    $.ajax({
        url: license,
        type: 'GET',
        xhrFields: {
          // The 'xhrFields' property sets additional fields on the XMLHttpRequest.
          // This can be used to set the 'withCredentials' property.
          // Set the value to 'true' if you'd like to pass cookies to the server.
          // If this is enabled, your server must respond with the header
          // 'Access-Control-Allow-Credentials: true'.
          withCredentials: false
        },
        success: function(data){ 
          licenseUrl = data.licenseAcquisitionUrl;
          if (drmtype === 'wv'){
            document.getElementById('customData').value = data;
          } else {
            document.getElementById('customData').value = data.token;
            document.getElementById('licenseUrl').value = data.licenseAcquisitionUrl;
          }
          toggleLoadBtn('enable');
        },
        error: function(data) {
            console.log('Error sending token request. Setting hardcoded values instead.');
            document.getElementById('mediaUrl').value = pr_content;
            licenseUrl = 'http://pr.test.expressplay.com/playready/RightsManager.asmx';
            document.getElementById('customData').value = 'AQAAAAOzWv4AAABgIPXrFQNri0QcDf9Ve707tOFhib1-Wd7eIpI7ZMvRTw3xZJbnKyWTYWjLVmzLuUY65EmEwGp_D-gPzVVBySFQHqWGZoVdZLqP3DXfDCLQCr3TnqmNrN2nD20O4dn-fMxZeKLjn5K2f_muz0f0Sw6aF89mxbo';
            toggleLoadBtn('enable');
        }
    });
  }

  if (is_chrome && (!chrome.cast || !chrome.cast.isAvailable)) {
    setTimeout(initializeCastApi, 1000);
    // Pre-populate with an example video and license for easy testing.
    changeService('pr');
  }
  else {
    console.log("THIS APP REQUIRES CHROME.");
  }

  function debugLog(message) {
    if (document.getElementById("debugmessage")) {
      var dw = document.getElementById("debugmessage");
      dw.innerHTML += '\n' + JSON.stringify(message);
      return message;
    }
  };

  function initializeCastApi() {
    var sessionRequest = new chrome.cast.SessionRequest(applicationID);
    var apiConfig = new chrome.cast.ApiConfig(sessionRequest,
      sessionListener,
      receiverListener);

    chrome.cast.initialize(apiConfig, onInitSuccess, onError);
  };

  function onInitSuccess() {
    debugLog("init success");
  }

  function onError(e) {
    debugLog(e);
  }

  function onSuccess(message) {
    debugLog(message);
  }

  function onStopAppSuccess() {
    debugLog('Session stopped');
  }

  function sessionListener(e) {
    debugLog('New session ID:' + e.sessionId);
    session = e;
    if (session.media.length != 0) {
      debugLog('Found ' + session.media.length + ' existing media sessions.');
      onMediaDiscovered('sessionListener', session.media[0]);
    }
    session.addMediaListener(
        onMediaDiscovered.bind(this, 'addMediaListener'));
    session.addUpdateListener(sessionUpdateListener.bind(this));  
    session.addMessageListener(MESSAGE_NAMESPACE, onReceiverMessage.bind(this));
  }

  function onReceiverMessage(namespace, message) {
    debugLog(namespace + ':' + message);
  }

  function sessionUpdateListener(isAlive) {
    var message = isAlive ? 'Session Updated' : 'Session Removed';
    message += ': ' + session.sessionId;
    debugLog(message);
    if (!isAlive) {
      session = null;
      var playpauseresume = document.getElementById("playpauseresume");
      if (playpauseresume) {
        playpauseresume.innerHTML = play;
      }
      if( timer ) {
        clearInterval(timer);
      }
    }
  }

  function receiverListener(e) {
    if (e === 'available') {
      debugLog("receiver found");
    } else {
      debugLog("receiver list empty");
    }
  }

  function launchApp() {
    debugLog("launching app...");
    chrome.cast.requestSession(onRequestSessionSuccess, onLaunchError);
  }

  function onRequestSessionSuccess(e) {
    debugLog("session success: " + e.sessionId);
    session = e;
    session.addMessageListener(MESSAGE_NAMESPACE, onReceiverMessage.bind(this));
  }

  function onLaunchError() {
    debugLog("launch error");
  }

  function stopApp() {
    session.stop(onStopAppSuccess, onError);
    if (timer) {
      clearInterval(timer);
    }
  }

  function loadMedia() {
    var mediaUrl = document.getElementById('mediaUrl').value;
    var custom = document.getElementById('customData').value;
    if (mediaUrl.length > 0) {
      loadMediaUrl(mediaUrl, custom);
    }
  }

  function loadMediaUrl(mediaURL, customData) {
    if (!session) {
      debugLog("no session");
      return;
    }
    var mediaInfo = new chrome.cast.media.MediaInfo(mediaURL);
    mediaInfo.contentType = 'video/mp4';
    var request = new chrome.cast.media.LoadRequest(mediaInfo);
    request.currentTime = initialTimeIndexSeconds;
    request.autoplay = true;
    request.sessionId = session.sessionId;
    if (customData) {
      if(drmtype === 'wv'){
        request.customData = {
         licenseUrl: document.getElementById('customData').value
        };
      } else if(drmtype === 'pr'){
        request.customData = {
         licenseCustomData: document.getElementById('customData').value,
         licenseUrl: document.getElementById('licenseUrl').value
        };
      }
    }
    else {
      //request.customData = {
      //  licenseCustomData: document.getElementById('customData').value,
      //  licenseUrl: 'https://expressplay-licensing.axprod.net/LicensingService.ashx'
      //}
      request.customData = null;
    }
    console.log(request);
    session.loadMedia(request,
      onMediaDiscovered.bind(this, 'loadMedia'),
      onMediaError);
  }

  function onMediaDiscovered(how, mediaSession) {
    debugLog("new media session ID:" + mediaSession.mediaSessionId + ' (' + how + ')');
    currentMedia = mediaSession;
    mediaSession.addUpdateListener(onMediaStatusUpdate);
    mediaCurrentTime = currentMedia.currentTime;
    playpauseresume.innerHTML = pause;
    if (!timer) {
      timer = setInterval(updateCurrentTime.bind(this), 1000);
    }
  }

  function onMediaError(e) {
    debugLog(e);
  }

  function onMediaStatusUpdate(isAlive) {
    if (progressFlag) {
      document.getElementById("progress").value = parseInt(100 * currentMedia.currentTime / currentMedia.media.duration);
      document.getElementById("mediaposition").innerHTML = currentMedia.currentTime;
      document.getElementById("mediaduration").innerHTML = currentMedia.media.duration;
    }
    document.getElementById("playerstate").innerHTML = currentMedia.playerState;
  }

  function playMedia() {
    if (!currentMedia) {
      return;
    } 

    if (timer) {
      clearInterval(timer);
    }

    var playpauseresume = document.getElementById("playpauseresume");
    if (playpauseresume.innerHTML.indexOf(play) > -1) {
      currentMedia.play(null,
        mediaCommandSuccessCallback.bind(this,"playing started for " + currentMedia.sessionId),
        onError);
        playpauseresume.innerHTML = pause;
        currentMedia.addUpdateListener(onMediaStatusUpdate);
        debugLog("play started");
        timer = setInterval(updateCurrentTime.bind(this), 1000);
    } else if (playpauseresume.innerHTML.indexOf(pause) > -1) {
      currentMedia.pause(null,
        mediaCommandSuccessCallback.bind(this,"paused " + currentMedia.sessionId),
        onError);
      playpauseresume.innerHTML = play;
      debugLog("paused");
    } else if (playpauseresume.innerHTML.indexOf(resume) > -1) {
      currentMedia.play(null,
        mediaCommandSuccessCallback.bind(this,"resumed " + currentMedia.sessionId),
        onError);
      playpauseresume.innerHTML = pause;
      debugLog("resumed");
      timer = setInterval(updateCurrentTime.bind(this), 1000);
    }
  }

  function stopMedia() {
    if (!currentMedia) {
      return;
    }

    currentMedia.stop(null,
      mediaCommandSuccessCallback.bind(this,"stopped " + currentMedia.sessionId),
      onError);
    var playpauseresume = document.getElementById("playpauseresume");
    playpauseresume.innerHTML = play;
    debugLog("media stopped");
    if (timer) {
      clearInterval(timer);
    }
  }

  function seekMedia(pos) {
    debugLog('Seeking ' + currentMedia.sessionId + ':' +
      currentMedia.mediaSessionId + ' to ' + pos + "%");
    progressFlag = 0;
    var request = new chrome.cast.media.SeekRequest();
    request.currentTime = pos * currentMedia.media.duration / 100;
    request.resumeState = chrome.cast.media.PlayerState.PAUSED; 
    currentMedia.seek(request,
      onSeekSuccess.bind(this, 'media seek done'),
      onError);
  }

  function onSeekSuccess(info) {
    debugLog(info);
    setTimeout(function(){progressFlag = 1},1500);
  }

  function mediaCommandSuccessCallback(info) {
    debugLog(info);
  }

  function updateCurrentTime() {
    if (!session || !currentMedia) {
      return;
    }

    if (currentMedia.media && currentMedia.media.duration != null) {
      var cTime = currentMedia.getEstimatedTime();
      document.getElementById("progress").value = parseInt(100 * cTime / currentMedia.media.duration);
      document.getElementById("mediaposition").innerHTML = cTime;
    } else {
      document.getElementById("progress").value = 0;
      document.getElementById("mediaposition").innerHTML = 0;
      if( timer ) {
        clearInterval(timer);
      }
    }
  }